@@ -6,8 +6,6 @@ from __future__ import print_function
 
 import numpy as np
 import re
-from scipy import linalg
-import scipy.ndimage as ndi
 from six.moves import range
 import os
 import threading
@@ -25,6 +23,12 @@ try:
     from PIL import Image as pil_image
 except ImportError:
     pil_image = None
+    ImageEnhance = None
+
+try:
+    import scipy
+except ImportError:
+    scipy = None
 
 if pil_image is not None:
     _PIL_INTERPOLATION_METHODS = {
@@ -144,7 +148,7 @@ def random_zoom(x, zoom_range, row_axis=1, col_axis=2, channel_axis=0,
     """
     if len(zoom_range) != 2:
         raise ValueError('`zoom_range` should be a tuple or list of two'
-                         ' floats. Received: ', zoom_range)
+                         ' floats. Received: ', (zoom_range,))
 
     if zoom_range[0] == 1 and zoom_range[1] == 1:
         zx, zy = 1, 1
@@ -208,6 +212,9 @@ def apply_brightness_shift(x, brightness):
     # Raises
         ValueError if `brightness_range` isn't a tuple.
     """
+    if ImageEnhance is None:
+        raise ImportError('Using brightness shifts requires PIL. '
+                          'Install PIL or Pillow.')
     x = array_to_img(x)
     x = imgenhancer_Brightness = ImageEnhance.Brightness(x)
     x = imgenhancer_Brightness.enhance(brightness)
@@ -232,7 +239,7 @@ def random_brightness(x, brightness_range):
     if len(brightness_range) != 2:
         raise ValueError(
             '`brightness_range should be tuple or list of two floats. '
-            'Received: %s' % brightness_range)
+            'Received: %s' % (brightness_range,))
 
     u = np.random.uniform(brightness_range[0], brightness_range[1])
     return apply_brightness_shift(x, u)
@@ -272,6 +279,9 @@ def apply_affine_transform(x, theta=0, tx=0, ty=0, shear=0, zx=1, zy=1,
     # Returns
         The transformed version of the input.
     """
+    if scipy is None:
+        raise ImportError('Image transformations require SciPy. '
+                          'Install SciPy.')
     transform_matrix = None
     if theta != 0:
         theta = np.deg2rad(theta)
@@ -316,7 +326,7 @@ def apply_affine_transform(x, theta=0, tx=0, ty=0, shear=0, zx=1, zy=1,
         final_affine_matrix = transform_matrix[:2, :2]
         final_offset = transform_matrix[:2, 2]
 
-        channel_images = [ndi.interpolation.affine_transform(
+        channel_images = [scipy.ndimage.interpolation.affine_transform(
             x_channel,
             final_affine_matrix,
             final_offset,
@@ -376,7 +386,10 @@ def array_to_img(x, data_format=None, scale=True):
         if x_max != 0:
             x /= x_max
         x *= 255
-    if x.shape[2] == 3:
+    if x.shape[2] == 4:
+        # RGBA
+        return pil_image.fromarray(x.astype('uint8'), 'RGBA')
+    elif x.shape[2] == 3:
         # RGB
         return pil_image.fromarray(x.astype('uint8'), 'RGB')
     elif x.shape[2] == 1:
@@ -441,16 +454,21 @@ def save_img(path,
         **kwargs: Additional keyword arguments passed to `PIL.Image.save()`.
     """
     img = array_to_img(x, data_format=data_format, scale=scale)
+    if img.mode == 'RGBA' and (file_format == 'jpg' or file_format == 'jpeg'):
+        warnings.warn('The JPG format does not support '
+                      'RGBA images, converting to RGB.')
+        img = img.convert('RGB')
     img.save(path, format=file_format, **kwargs)
 
 
-def load_img(path, grayscale=False, target_size=None,
+def load_img(path, grayscale=False, color_mode='rgb', target_size=None,
              interpolation='nearest'):
     """Loads an image into PIL format.
 
     # Arguments
         path: Path to image file.
-        grayscale: Boolean, whether to load the image as grayscale.
+        color_mode: One of "grayscale", "rbg", "rgba". Default: "rgb".
+            The desired image format.
         target_size: Either `None` (default to original size)
             or tuple of ints `(img_height, img_width)`.
         interpolation: Interpolation method used to resample the image if the
@@ -467,16 +485,25 @@ def load_img(path, grayscale=False, target_size=None,
         ImportError: if PIL is not available.
         ValueError: if interpolation method is not supported.
     """
+    if grayscale is True:
+        warnings.warn('grayscale is deprecated. Please use '
+                      'color_mode = "grayscale"')
+        color_mode = 'grayscale'
     if pil_image is None:
         raise ImportError('Could not import PIL.Image. '
                           'The use of `array_to_img` requires PIL.')
     img = pil_image.open(path)
-    if grayscale:
+    if color_mode == 'grayscale':
         if img.mode != 'L':
             img = img.convert('L')
-    else:
+    elif color_mode == 'rgba':
+        if img.mode != 'RGBA':
+            img = img.convert('RGBA')
+    elif color_mode == 'rgb':
         if img.mode != 'RGB':
             img = img.convert('RGB')
+    else:
+        raise ValueError('color_mode must be "grayscale", "rbg", or "rgba"')
     if target_size is not None:
         width_height_tuple = (target_size[1], target_size[0])
         if img.size != width_height_tuple:
@@ -531,6 +558,8 @@ class ImageDataGenerator(object):
                 same as with `height_shift_range=[-1, 0, +1]`,
                 while with `height_shift_range=1.0` possible values are floats
                 in the interval [-1.0, +1.0).
+        brightness_range: Tuple or list of two floats. Range for picking
+            a brightness shift value from.
         shear_range: Float. Shear Intensity
             (Shear angle in counter-clockwise direction in degrees)
         zoom_range: Float or [lower, upper]. Range for random zoom.
@@ -552,7 +581,7 @@ class ImageDataGenerator(object):
         rescale: rescaling factor. Defaults to None.
             If None or 0, no rescaling is applied,
             otherwise we multiply the data by the value provided
-            (before applying any other transformation).
+            (after applying all other transformations).
         preprocessing_function: function that will be implied on each input.
             The function will run after the image is resized and augmented.
             The function should take one argument:
@@ -643,7 +672,7 @@ class ImageDataGenerator(object):
     # we create two instances with the same arguments
     data_gen_args = dict(featurewise_center=True,
                          featurewise_std_normalization=True,
-                         rotation_range=90.,
+                         rotation_range=90,
                          width_shift_range=0.1,
                          height_shift_range=0.1,
                          zoom_range=0.2)
@@ -682,7 +711,7 @@ class ImageDataGenerator(object):
                  samplewise_std_normalization=False,
                  zca_whitening=False,
                  zca_epsilon=1e-6,
-                 rotation_range=0.,
+                 rotation_range=0,
                  width_shift_range=0.,
                  height_shift_range=0.,
                  brightness_range=None,
@@ -751,7 +780,7 @@ class ImageDataGenerator(object):
         else:
             raise ValueError('`zoom_range` should be a float or '
                              'a tuple or list of two floats. '
-                             'Received: %s' % zoom_range)
+                             'Received: %s' % (zoom_range,))
         if zca_whitening:
             if not featurewise_center:
                 self.featurewise_center = True
@@ -795,8 +824,9 @@ class ImageDataGenerator(object):
                 Can be used to feed the model miscellaneous data
                 along with the images.
                 In case of grayscale data, the channels axis of the image array
-                should have value 1, and in case
-                of RGB data, it should have value 3.
+                should have value 1, in case
+                of RGB data, it should have value 3, and in case
+                of RGBA data, it should have value 4.
             y: Labels.
             batch_size: Int (default: 32).
             shuffle: Boolean (default: True).
@@ -855,15 +885,14 @@ class ImageDataGenerator(object):
                 inside each of the subdirectories directory tree
                 will be included in the generator.
                 See [this script](
-                    https://gist.github.com/fchollet/
-                    0830affa1f7f19fd47b06d4cf89ed44d)
+                https://gist.github.com/fchollet/0830affa1f7f19fd47b06d4cf89ed44d)
                 for more details.
             target_size: Tuple of integers `(height, width)`,
                 default: `(256, 256)`.
                 The dimensions to which all images found will be resized.
-            color_mode: One of "grayscale", "rbg". Default: "rgb".
+            color_mode: One of "grayscale", "rbg", "rgba". Default: "rgb".
                 Whether the images will be converted to
-                have 1 or 3 color channels.
+                have 1, 3, or 4 channels.
             classes: Optional list of class subdirectories
                 (e.g. `['dogs', 'cats']`). Default: None.
                 If not provided, the list of classes will be automatically
@@ -1058,7 +1087,7 @@ class ImageDataGenerator(object):
             if len(self.brightness_range) != 2:
                 raise ValueError(
                     '`brightness_range should be tuple or list of two floats. '
-                    'Received: %s' % brightness_range)
+                    'Received: %s' % (self.brightness_range,))
             brightness = np.random.uniform(self.brightness_range[0],
                                            self.brightness_range[1])
 
@@ -1096,7 +1125,7 @@ class ImageDataGenerator(object):
                 - `'brightness'`: Float. Brightness shift intensity.
 
         # Returns
-            A ransformed version of the input (same shape).
+            A transformed version of the input (same shape).
         """
         # x is a single image, so it doesn't have image number at index 0
         img_row_axis = self.row_axis - 1
@@ -1157,8 +1186,9 @@ class ImageDataGenerator(object):
         # Arguments
             x: Sample data. Should have rank 4.
              In case of grayscale data,
-             the channels axis should have value 1, and in case
-             of RGB data, it should have value 3.
+             the channels axis should have value 1, in case
+             of RGB data, it should have value 3, and in case
+             of RGBA data, it should have value 4.
             augment: Boolean (default: False).
                 Whether to fit on randomly augmented samples.
             rounds: Int (default: 1).
@@ -1210,10 +1240,13 @@ class ImageDataGenerator(object):
             x /= (self.std + backend.epsilon())
 
         if self.zca_whitening:
+            if scipy is None:
+                raise ImportError('Using zca_whitening requires SciPy. '
+                                  'Install SciPy.')
             flat_x = np.reshape(
                 x, (x.shape[0], x.shape[1] * x.shape[2] * x.shape[3]))
             sigma = np.dot(flat_x.T, flat_x) / flat_x.shape[0]
-            u, s, _ = linalg.svd(sigma)
+            u, s, _ = scipy.linalg.svd(sigma)
             s_inv = 1. / np.sqrt(s[np.newaxis] + self.zca_epsilon)
             self.principal_components = (u * s_inv).dot(u.T)
 
@@ -1397,7 +1430,7 @@ class NumpyArrayIterator(Iterator):
             warnings.warn('NumpyArrayIterator is set to use the '
                           'data format convention "' + data_format + '" '
                           '(channels on axis ' + str(channels_axis) +
-                          '), i.e. expected either 1, 3 or 4 '
+                          '), i.e. expected either 1, 3, or 4 '
                           'channels on axis ' + str(channels_axis) + '. '
                           'However, it was passed an array with shape ' +
                           str(self.x.shape) + ' (' +
@@ -1583,7 +1616,8 @@ class DirectoryIterator(Iterator):
         image_data_generator: Instance of `ImageDataGenerator`
             to use for random transformations and normalization.
         target_size: tuple of integers, dimensions to resize input images to.
-        color_mode: One of `"rgb"`, `"grayscale"`. Color mode to read images.
+        color_mode: One of `"rgb"`, `"rgba"`, `"grayscale"`.
+            Color mode to read images.
         classes: Optional list of strings, names of subdirectories
             containing images from each class (e.g. `["dogs", "cats"]`).
             It will be computed automatically if not set.
@@ -1630,12 +1664,17 @@ class DirectoryIterator(Iterator):
         self.directory = directory
         self.image_data_generator = image_data_generator
         self.target_size = tuple(target_size)
-        if color_mode not in {'rgb', 'grayscale'}:
+        if color_mode not in {'rgb', 'rgba', 'grayscale'}:
             raise ValueError('Invalid color mode:', color_mode,
-                             '; expected "rgb" or "grayscale".')
+                             '; expected "rgb", "rgba", or "grayscale".')
         self.color_mode = color_mode
         self.data_format = data_format
-        if self.color_mode == 'rgb':
+        if self.color_mode == 'rgba':
+            if self.data_format == 'channels_last':
+                self.image_shape = self.target_size + (4,)
+            else:
+                self.image_shape = (4,) + self.target_size
+        elif self.color_mode == 'rgb':
             if self.data_format == 'channels_last':
                 self.image_shape = self.target_size + (3,)
             else:
@@ -1724,15 +1763,18 @@ class DirectoryIterator(Iterator):
         batch_x = np.zeros(
             (len(index_array),) + self.image_shape,
             dtype=backend.floatx())
-        grayscale = self.color_mode == 'grayscale'
         # build batch of image data
         for i, j in enumerate(index_array):
             fname = self.filenames[j]
             img = load_img(os.path.join(self.directory, fname),
-                           grayscale=grayscale,
+                           color_mode=self.color_mode,
                            target_size=self.target_size,
                            interpolation=self.interpolation)
             x = img_to_array(img, data_format=self.data_format)
+            # Pillow images should be closed after `load_img`,
+            # but not PIL images.
+            if hasattr(img, 'close'):
+                img.close()
             params = self.image_data_generator.get_random_transform(x.shape)
             x = self.image_data_generator.apply_transform(x, params)
             x = self.image_data_generator.standardize(x)
